from taskiq import TaskiqScheduler


class MyTaskiqScheduler(TaskiqScheduler):
    async def startup(self) -> None:
        await super().startup()

    async def shutdown(self) -> None:
        await super().shutdown()



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
import time
from typing import Any
from uuid import uuid4

from fastapi import FastAPI
from starlette.datastructures import MutableHeaders
from starlette.types import Message, Receive, Scope, Send

from .context import get_correlation_id, set_correlation_id
from .log_filter import CorrelationIdFilter, EnsureValidExtraFilter


class RequestLoggingMiddleware:
    def __init__(
        self,
        app: FastAPI,
        header_name: str = "X-Request-ID",
        to_logger: str = "omnichannel-console",
    ):
        self.app = app
        self.header_name = header_name
        self.logger = logging.getLogger(to_logger)
        if not any(isinstance(f, CorrelationIdFilter) for f in self.logger.filters):
            self.logger.addFilter(CorrelationIdFilter())

        if not any(isinstance(f, EnsureValidExtraFilter) for f in self.logger.filters):
            self.logger.addFilter(EnsureValidExtraFilter())

        self._change_uvicorn_fastapi_log_level()

    def _change_uvicorn_fastapi_log_level(self, level: int = logging.ERROR):
        """Change uvicorn and fastapi log level to reduce noise in logs"""
        uvicorn_loggers = ["uvicorn", "uvicorn.error", "uvicorn.access", "fastapi"]
        for logger_name in uvicorn_loggers:
            logger = logging.getLogger(logger_name)
            logger.setLevel(level)

    def _get_client_ip(self, scope: Scope) -> str:
        client = scope.get("client")
        if client:
            return client[0]

        # Check for forwarded headers
        headers = dict(scope.get("headers", []))
        try:
            x_forwarded_for = headers.get(b"x-forwarded-for")
            if x_forwarded_for:
                return x_forwarded_for.decode().split(",")[0].strip()

            x_real_ip = headers.get(b"x-real-ip")
            if x_real_ip:
                return x_real_ip.decode()
            return "unknown"
        except Exception as e:
            return f"unknown-exc-{e}"

    def _get_request_path(self, scope: Scope, with_params: bool = False) -> str:
        path = scope.get("path", "")
        if with_params:
            query_string = scope.get("query_string", b"").decode()
            if query_string:
                return f"{path}?{query_string}"
        return path

    def _get_request_method(self, scope: Scope) -> str:
        return scope.get("method", "unknown")

    async def __call__(self, scope: Scope, receive: Receive, send: Send) -> Any:
        """
        Main middleware function to capture request/response data and log it
        """
        if scope["type"] not in ("http", "websocket"):
            await self.app(scope, receive, send)
            return

        start_time = time.time_ns()
        status_code = 0
        headers = MutableHeaders(scope=scope)
        header_value = headers.get(self.header_name.lower())
        id_value = header_value if header_value else str(uuid4())
        set_correlation_id(id_value)

        em_group_id = headers.get("x-em-group-id")

        async def send_wrapper(message: Message) -> None:
            nonlocal status_code
            if message["type"] == "http.response.start":
                status_code = message.get("status", -1)
                resp_cid = get_correlation_id()
                if resp_cid:
                    message_headers = MutableHeaders(scope=message)
                    message_headers.append(self.header_name, resp_cid)

            await send(message)

        try:
            await self.app(scope, receive, send_wrapper)
        finally:
            req_url = self._get_request_path(scope, with_params=True)
            req_resp_time = (time.time_ns() - start_time) / 1_000_000  # in ms
            req_resp_extra = {
                "req_client_ip": self._get_client_ip(scope),
                "req_url": req_url,
                "req_method": self._get_request_method(scope),
                "resp_status": status_code,
                "loc": "core0000",
            }
            if em_group_id:
                req_resp_extra["em_group_id"] = em_group_id

            self.logger.info(
                f"api request {req_url} {status_code} {req_resp_time}ms",
                extra=req_resp_extra,
            )



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from typing import Optional

from pydantic_settings import BaseSettings


class RedisSettings(BaseSettings):
    REDIS_URL: Optional[str] = "mongodb://localhost:27017"



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

import logging
from typing import Dict, List, Optional

from fastapi import APIRouter, FastAPI

logger = logging.getLogger("console")


def create_fastapi_api_app(
    service_name: str,
    service_path: str,
    routers: List[APIRouter],
    custom_exc_handlers: Optional[Dict],
    middlewares: List = [],
    docs_url: str = "/docs",
    openapi_url: str = "/openapi.json",
):
    api_app = FastAPI(
        title=f"Service {service_name}",
        middleware=middlewares,
        docs_url=docs_url,
        openapi_url=openapi_url,
    )

    @api_app.get("/")
    async def root():
        return {"location": f"/{service_path}/api"}

    # add routers
    for x_router in routers:
        if x_router and isinstance(x_router, APIRouter):
            api_app.include_router(x_router)
            continue
        logger.warning(
            f"routers expected an instance of APIRouter but get {type(x_router)}"
        )

    # add exception handlers
    if custom_exc_handlers and isinstance(custom_exc_handlers, dict):
        for exc_class, exc_handler in custom_exc_handlers.items():
            if issubclass(exc_class, Exception) and callable(exc_handler):
                api_app.add_exception_handler(exc_class, exc_handler)
                logger.info(
                    f"add exception handler={exc_handler} for exception {exc_class}"
                )
                continue
            logger.warning(f"unexpected handler {exc_class=} | {custom_exc_handlers=}")

    return api_app



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from .context import get_correlation_id, set_correlation_id
from .fastapi_middleware import RequestLoggingMiddleware
from .installer import create_json_logger
from .log_filter import CorrelationIdFilter, EnsureValidExtraFilter

__all__ = [
    "RequestLoggingMiddleware",
    "CorrelationIdFilter",
    "EnsureValidExtraFilter",
    "get_correlation_id",
    "set_correlation_id",
    "create_json_logger",
]



# -*- coding: utf-8 -*-
import os

from taskiq import AsyncBroker
from taskiq_aio_pika import AioPikaBroker

from .mongo_result_backend import MongoDBResultBackend


def new_taskiq_broker(
    broker_rabbitmq_url: str,
    result_mongo_url: str,
    result_mongo_db: str = "taskiq",
    result_mongo_collection: str = "results",
    broker_qos: int = os.cpu_count(),
    middlewares: list = [],
) -> AioPikaBroker:
    result_backend = MongoDBResultBackend(
        mongo_url=result_mongo_url,
        database_name=result_mongo_db,
        collection_name=result_mongo_collection,
    )

    broker: AsyncBroker = AioPikaBroker(
        url=broker_rabbitmq_url,
        qos=broker_qos,
    ).with_result_backend(result_backend)

    if middlewares:
        broker.add_middlewares(*middlewares)

    return broker



# -*- coding: utf-8 -*-
import json
from datetime import date, datetime
from decimal import Decimal
from typing import Any
from uuid import UUID

from taskiq.abc.serializer import TaskiqSerializer


def _json_default(value: Any):
    """Make datetime (and friends) JSON serializable."""
    if isinstance(value, (datetime, date)):
        return value.isoformat()
    if isinstance(value, (UUID,)):
        return str(value)
    if isinstance(value, Decimal):
        return str(value)
    return str(value)


class DateTimeSafeJsonSerializer(TaskiqSerializer):
    """Json serializer that encodes datetime instances as ISO strings."""

    def dumpb(self, message: Any) -> bytes:
        return json.dumps(
            message,
            default=_json_default,
            separators=(",", ":"),
        ).encode("utf-8")

    def loadb(self, data: bytes) -> Any:
        return json.loads(data.decode("utf-8"))



# -*- coding: utf-8 -*-
# author: NhanDD3 <hp.duongducnhan@gmail.com>

from typing import Optional

from pydantic_settings import BaseSettings


class OpenTelemetrySettings(BaseSettings):
    OTEL_ENABLE: Optional[bool] = False
    OTEL_ENDPOINT: Optional[str] = "http://otel-collector-hni01.ftel.scc/v1/traces"
    OTEL_SERVICE_NAME: Optional[str] = "EasyCore"
    OTEL_ENVIRONMENT: Optional[str] = "dev"



# -*- coding: utf-8 -*-
from typing import Any

from taskiq import TaskiqMessage, TaskiqMiddleware, TaskiqResult


class LoggingMiddlewareWorker(TaskiqMiddleware):

    async def pre_execute(self, message: TaskiqMessage) -> TaskiqMessage:
        # logger.info(
        #     f"Preparing to execute task: {message.task_name}",
        #     extra={
        #         "task_id": message.task_id,
        #         "task_args": str(message.args),
        #         "task_kwargs": str(message.kwargs),
        #     },
        # )
        return message

    async def post_execute(
        self, message: TaskiqMessage, result: TaskiqResult[Any]
    ) -> None:
        # Ghi log kết quả sau khi thực thi
        # logger.info(
        #     f"Task {message.task_name} completed with result: {result.return_value}"
        # )
        result.labels["task_name"] = message.task_name
